/* arch/arm/mach-s5pc100/include/mach/gpio.h
 *
 * Copyright 2008 Openmoko, Inc.
 * Copyright 2008 Simtec Electronics
 *	http://armlinux.simtec.co.uk/
 *	Ben Dooks <ben@simtec.co.uk>
 *
 * S5PC100 - GPIO lib support
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/

#define gpio_get_value	__gpio_get_value
#define gpio_set_value	__gpio_set_value
#define gpio_cansleep	__gpio_cansleep
#define gpio_to_irq	__gpio_to_irq

/* GPIO bank sizes */
#if defined(CONFIG_CPU_S5PC100)
#define S5PC1XX_GPIO_A0_NR	(8)
#define S5PC1XX_GPIO_A1_NR	(5)
#define S5PC1XX_GPIO_B_NR	(8)
#define S5PC1XX_GPIO_C_NR	(5)
#define S5PC1XX_GPIO_D_NR	(7)
#define S5PC1XX_GPIO_E0_NR	(8)
#define S5PC1XX_GPIO_E1_NR	(6)
#define S5PC1XX_GPIO_F0_NR	(8)
#define S5PC1XX_GPIO_F1_NR	(8)
#define S5PC1XX_GPIO_F2_NR	(8)
#define S5PC1XX_GPIO_F3_NR	(4)
#define S5PC1XX_GPIO_G0_NR	(8)
#define S5PC1XX_GPIO_G1_NR	(3)
#define S5PC1XX_GPIO_G2_NR	(7)
#define S5PC1XX_GPIO_G3_NR	(7)
#define S5PC1XX_GPIO_H0_NR	(8)
#define S5PC1XX_GPIO_H1_NR	(8)
#define S5PC1XX_GPIO_H2_NR	(8)
#define S5PC1XX_GPIO_H3_NR	(8)
#define S5PC1XX_GPIO_I_NR	(8)
#define S5PC1XX_GPIO_J0_NR	(8)
#define S5PC1XX_GPIO_J1_NR	(5)
#define S5PC1XX_GPIO_J2_NR	(8)
#define S5PC1XX_GPIO_J3_NR	(8)
#define S5PC1XX_GPIO_J4_NR	(4)
#define S5PC1XX_GPIO_K0_NR	(8)
#define S5PC1XX_GPIO_K1_NR	(6)
#define S5PC1XX_GPIO_K2_NR	(8)
#define S5PC1XX_GPIO_K3_NR	(8)
#define S5PC1XX_GPIO_L0_NR	(8)
#define S5PC1XX_GPIO_L1_NR	(8)
#define S5PC1XX_GPIO_L2_NR	(8)
#define S5PC1XX_GPIO_L3_NR	(8)
#define S5PC1XX_GPIO_L4_NR	(5)

/* GPIO bank numbes */

/* CONFIG_S3C_GPIO_SPACE allows the user to select extra
 * space for debugging purposes so that any accidental
 * change from one gpio bank to another can be caught.
*/

#define S5PC1XX_GPIO_NEXT(__gpio) \
	((__gpio##_START) + (__gpio##_NR) + CONFIG_S3C_GPIO_SPACE + 1)

enum s3c_gpio_number {
	S5PC1XX_GPIO_A0_START 	= 0,
	S5PC1XX_GPIO_A1_START 	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_A0),
	S5PC1XX_GPIO_B_START 	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_A1),
	S5PC1XX_GPIO_C_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_B),
	S5PC1XX_GPIO_D_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_C),
	S5PC1XX_GPIO_E0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_D),
	S5PC1XX_GPIO_E1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_E0),
	S5PC1XX_GPIO_F0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_E1),
	S5PC1XX_GPIO_F1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_F0),
	S5PC1XX_GPIO_F2_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_F1),
	S5PC1XX_GPIO_F3_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_F2),
	S5PC1XX_GPIO_G0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_F3),
	S5PC1XX_GPIO_G1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_G0),
	S5PC1XX_GPIO_G2_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_G1),
	S5PC1XX_GPIO_G3_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_G2),
	S5PC1XX_GPIO_H0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_G3),
	S5PC1XX_GPIO_H1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_H0),
	S5PC1XX_GPIO_H2_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_H1),
	S5PC1XX_GPIO_H3_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_H2),
	S5PC1XX_GPIO_I_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_H3),
	S5PC1XX_GPIO_J0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_I),
	S5PC1XX_GPIO_J1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_J0),
	S5PC1XX_GPIO_J2_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_J1),
	S5PC1XX_GPIO_J3_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_J2),
	S5PC1XX_GPIO_J4_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_J3),
	S5PC1XX_GPIO_K0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_J4),
	S5PC1XX_GPIO_K1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_K0),
	S5PC1XX_GPIO_K2_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_K1),
	S5PC1XX_GPIO_K3_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_K2),
	S5PC1XX_GPIO_L0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_K3),
	S5PC1XX_GPIO_L1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_L0),
	S5PC1XX_GPIO_L2_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_L1),
	S5PC1XX_GPIO_L3_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_L2),
	S5PC1XX_GPIO_L4_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_L3),
};

/* S5PC1XX GPIO number definitions. */
#define S5PC1XX_GPA0(_nr)	(S5PC1XX_GPIO_A0_START + (_nr))
#define S5PC1XX_GPA1(_nr)	(S5PC1XX_GPIO_A1_START + (_nr))
#define S5PC1XX_GPB(_nr)	(S5PC1XX_GPIO_B_START + (_nr))
#define S5PC1XX_GPC(_nr)	(S5PC1XX_GPIO_C_START + (_nr))
#define S5PC1XX_GPD(_nr)	(S5PC1XX_GPIO_D_START + (_nr))
#define S5PC1XX_GPE0(_nr)	(S5PC1XX_GPIO_E0_START + (_nr))
#define S5PC1XX_GPE1(_nr)	(S5PC1XX_GPIO_E1_START + (_nr))
#define S5PC1XX_GPF0(_nr)	(S5PC1XX_GPIO_F0_START + (_nr))
#define S5PC1XX_GPF1(_nr)	(S5PC1XX_GPIO_F1_START + (_nr))
#define S5PC1XX_GPF2(_nr)	(S5PC1XX_GPIO_F2_START + (_nr))
#define S5PC1XX_GPF3(_nr)	(S5PC1XX_GPIO_F3_START + (_nr))
#define S5PC1XX_GPG0(_nr)	(S5PC1XX_GPIO_G0_START + (_nr))
#define S5PC1XX_GPG1(_nr)	(S5PC1XX_GPIO_G1_START + (_nr))
#define S5PC1XX_GPG2(_nr)	(S5PC1XX_GPIO_G2_START + (_nr))
#define S5PC1XX_GPG3(_nr)	(S5PC1XX_GPIO_G3_START + (_nr))
#define S5PC1XX_GPH0(_nr)	(S5PC1XX_GPIO_H0_START + (_nr))
#define S5PC1XX_GPH1(_nr)	(S5PC1XX_GPIO_H1_START + (_nr))
#define S5PC1XX_GPH2(_nr)	(S5PC1XX_GPIO_H2_START + (_nr))
#define S5PC1XX_GPH3(_nr)	(S5PC1XX_GPIO_H3_START + (_nr))
#define S5PC1XX_GPI(_nr)	(S5PC1XX_GPIO_I_START + (_nr))
#define S5PC1XX_GPJ0(_nr)	(S5PC1XX_GPIO_J0_START + (_nr))
#define S5PC1XX_GPJ1(_nr)	(S5PC1XX_GPIO_J1_START + (_nr))
#define S5PC1XX_GPJ2(_nr)	(S5PC1XX_GPIO_J2_START + (_nr))
#define S5PC1XX_GPJ3(_nr)	(S5PC1XX_GPIO_J3_START + (_nr))
#define S5PC1XX_GPJ4(_nr)	(S5PC1XX_GPIO_J4_START + (_nr))
#define S5PC1XX_GPK0(_nr)	(S5PC1XX_GPIO_K0_START + (_nr))
#define S5PC1XX_GPK1(_nr)	(S5PC1XX_GPIO_K1_START + (_nr))
#define S5PC1XX_GPK2(_nr)	(S5PC1XX_GPIO_K2_START + (_nr))
#define S5PC1XX_GPK3(_nr)	(S5PC1XX_GPIO_K3_START + (_nr))
#define S5PC1XX_GPL0(_nr)	(S5PC1XX_GPIO_L0_START + (_nr))
#define S5PC1XX_GPL1(_nr)	(S5PC1XX_GPIO_L1_START + (_nr))
#define S5PC1XX_GPL2(_nr)	(S5PC1XX_GPIO_L2_START + (_nr))
#define S5PC1XX_GPL3(_nr)	(S5PC1XX_GPIO_L3_START + (_nr))
#define S5PC1XX_GPL4(_nr)	(S5PC1XX_GPIO_L4_START + (_nr))

/* the end of the S5PC1XX specific gpios */
#define S5PC1XX_GPIO_END	(S5PC1XX_GPL4(S5PC1XX_GPIO_L4_NR) + 1)
#define S3C_GPIO_END		S5PC1XX_GPIO_END

/* define the number of gpios we need to the one after the MP04() range */
#define ARCH_NR_GPIOS	(S5PC1XX_GPL4(S5PC1XX_GPIO_L4_NR) + 1)

#elif defined(CONFIG_CPU_S5PC110)
#define S5PC1XX_GPIO_A0_NR	(8)
#define S5PC1XX_GPIO_A1_NR	(4)
#define S5PC1XX_GPIO_B_NR	(8)
#define S5PC1XX_GPIO_C0_NR	(5)
#define S5PC1XX_GPIO_C1_NR	(5)
#define S5PC1XX_GPIO_D0_NR	(4)
#define S5PC1XX_GPIO_D1_NR	(6)
#define S5PC1XX_GPIO_E0_NR	(8)
#define S5PC1XX_GPIO_E1_NR	(5)
#define S5PC1XX_GPIO_F0_NR	(8)
#define S5PC1XX_GPIO_F1_NR	(8)
#define S5PC1XX_GPIO_F2_NR	(8)
#define S5PC1XX_GPIO_F3_NR	(6)
#define S5PC1XX_GPIO_G0_NR	(7)
#define S5PC1XX_GPIO_G1_NR	(7)
#define S5PC1XX_GPIO_G2_NR	(7)
#define S5PC1XX_GPIO_G3_NR	(7)
#define S5PC1XX_GPIO_H0_NR	(8)
#define S5PC1XX_GPIO_H1_NR	(8)
#define S5PC1XX_GPIO_H2_NR	(8)
#define S5PC1XX_GPIO_H3_NR	(8)
#define S5PC1XX_GPIO_I_NR	(7)
#define S5PC1XX_GPIO_J0_NR	(8)
#define S5PC1XX_GPIO_J1_NR	(6)
#define S5PC1XX_GPIO_J2_NR	(8)
#define S5PC1XX_GPIO_J3_NR	(8)
#define S5PC1XX_GPIO_J4_NR	(5)

#define S5PC1XX_GPIO_MP01_NR	(8)
#define S5PC1XX_GPIO_MP02_NR	(4)
#define S5PC1XX_GPIO_MP03_NR	(8)
#define S5PC1XX_GPIO_MP04_NR	(8)
#define S5PC1XX_GPIO_MP05_NR	(8)
#define S5PC1XX_GPIO_MP06_NR	(8)
#define S5PC1XX_GPIO_MP07_NR	(8)

#define S5PC1XX_GPIO_MP10_NR	(8)
#define S5PC1XX_GPIO_MP11_NR	(8)
#define S5PC1XX_GPIO_MP12_NR	(8)
#define S5PC1XX_GPIO_MP13_NR	(8)
#define S5PC1XX_GPIO_MP14_NR	(8)
#define S5PC1XX_GPIO_MP15_NR	(8)
#define S5PC1XX_GPIO_MP16_NR	(8)
#define S5PC1XX_GPIO_MP17_NR	(8)
#define S5PC1XX_GPIO_MP18_NR	(7)

#define S5PC1XX_GPIO_MP20_NR	(8)
#define S5PC1XX_GPIO_MP21_NR	(8)
#define S5PC1XX_GPIO_MP22_NR	(8)
#define S5PC1XX_GPIO_MP23_NR	(8)
#define S5PC1XX_GPIO_MP24_NR	(8)
#define S5PC1XX_GPIO_MP25_NR	(8)
#define S5PC1XX_GPIO_MP26_NR	(8)
#define S5PC1XX_GPIO_MP27_NR	(8)
#define S5PC1XX_GPIO_MP28_NR	(7)

/* GPIO bank numbes */

/* CONFIG_S3C_GPIO_SPACE allows the user to select extra
 * space for debugging purposes so that any accidental
 * change from one gpio bank to another can be caught.
*/

#define S5PC1XX_GPIO_NEXT(__gpio) \
	((__gpio##_START) + (__gpio##_NR) + CONFIG_S3C_GPIO_SPACE + 1)

enum s3c_gpio_number {
	S5PC1XX_GPIO_A0_START 	= 0,
	S5PC1XX_GPIO_A1_START 	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_A0),
	S5PC1XX_GPIO_B_START 	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_A1),
	S5PC1XX_GPIO_C0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_B),
	S5PC1XX_GPIO_C1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_C0),
	S5PC1XX_GPIO_D0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_C1),
	S5PC1XX_GPIO_D1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_D0),
	S5PC1XX_GPIO_E0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_D1),
	S5PC1XX_GPIO_E1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_E0),
	S5PC1XX_GPIO_F0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_E1),
	S5PC1XX_GPIO_F1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_F0),
	S5PC1XX_GPIO_F2_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_F1),
	S5PC1XX_GPIO_F3_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_F2),
	S5PC1XX_GPIO_G0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_F3),
	S5PC1XX_GPIO_G1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_G0),
	S5PC1XX_GPIO_G2_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_G1),
	S5PC1XX_GPIO_G3_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_G2),
	S5PC1XX_GPIO_H0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_G3),
	S5PC1XX_GPIO_H1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_H0),
	S5PC1XX_GPIO_H2_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_H1),
	S5PC1XX_GPIO_H3_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_H2),
	S5PC1XX_GPIO_I_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_H3),
	S5PC1XX_GPIO_J0_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_I),
	S5PC1XX_GPIO_J1_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_J0),
	S5PC1XX_GPIO_J2_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_J1),
	S5PC1XX_GPIO_J3_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_J2),
	S5PC1XX_GPIO_J4_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_J3),
	S5PC1XX_GPIO_MP01_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_J4),
	S5PC1XX_GPIO_MP02_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP01),
	S5PC1XX_GPIO_MP03_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP02),
	S5PC1XX_GPIO_MP04_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP03),
	S5PC1XX_GPIO_MP05_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP04),
	S5PC1XX_GPIO_MP06_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP05),
	S5PC1XX_GPIO_MP07_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP06),
	S5PC1XX_GPIO_MP10_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP07),
	S5PC1XX_GPIO_MP11_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP10),
	S5PC1XX_GPIO_MP12_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP11),
	S5PC1XX_GPIO_MP13_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP12),
	S5PC1XX_GPIO_MP14_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP13),
	S5PC1XX_GPIO_MP15_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP14),
	S5PC1XX_GPIO_MP16_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP15),
	S5PC1XX_GPIO_MP17_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP16),
	S5PC1XX_GPIO_MP18_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP17),	
	S5PC1XX_GPIO_MP20_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP18),
	S5PC1XX_GPIO_MP21_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP20),
	S5PC1XX_GPIO_MP22_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP21),
	S5PC1XX_GPIO_MP23_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP22),
	S5PC1XX_GPIO_MP24_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP23),
	S5PC1XX_GPIO_MP25_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP24),
	S5PC1XX_GPIO_MP26_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP25),
	S5PC1XX_GPIO_MP27_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP26),	
	S5PC1XX_GPIO_MP28_START	= S5PC1XX_GPIO_NEXT(S5PC1XX_GPIO_MP27),		

};

/* S5PC1XX GPIO number definitions. */
#define S5PC1XX_GPA0(_nr)	(S5PC1XX_GPIO_A0_START + (_nr))
#define S5PC1XX_GPA1(_nr)	(S5PC1XX_GPIO_A1_START + (_nr))
#define S5PC1XX_GPB(_nr)	(S5PC1XX_GPIO_B_START + (_nr))
#define S5PC1XX_GPC0(_nr)	(S5PC1XX_GPIO_C0_START + (_nr))
#define S5PC1XX_GPC1(_nr)	(S5PC1XX_GPIO_C1_START + (_nr))
#define S5PC1XX_GPD0(_nr)	(S5PC1XX_GPIO_D0_START + (_nr))
#define S5PC1XX_GPD1(_nr)	(S5PC1XX_GPIO_D1_START + (_nr))
#define S5PC1XX_GPE0(_nr)	(S5PC1XX_GPIO_E0_START + (_nr))
#define S5PC1XX_GPE1(_nr)	(S5PC1XX_GPIO_E1_START + (_nr))
#define S5PC1XX_GPF0(_nr)	(S5PC1XX_GPIO_F0_START + (_nr))
#define S5PC1XX_GPF1(_nr)	(S5PC1XX_GPIO_F1_START + (_nr))
#define S5PC1XX_GPF2(_nr)	(S5PC1XX_GPIO_F2_START + (_nr))
#define S5PC1XX_GPF3(_nr)	(S5PC1XX_GPIO_F3_START + (_nr))
#define S5PC1XX_GPG0(_nr)	(S5PC1XX_GPIO_G0_START + (_nr))
#define S5PC1XX_GPG1(_nr)	(S5PC1XX_GPIO_G1_START + (_nr))
#define S5PC1XX_GPG2(_nr)	(S5PC1XX_GPIO_G2_START + (_nr))
#define S5PC1XX_GPG3(_nr)	(S5PC1XX_GPIO_G3_START + (_nr))
#define S5PC1XX_GPH0(_nr)	(S5PC1XX_GPIO_H0_START + (_nr))
#define S5PC1XX_GPH1(_nr)	(S5PC1XX_GPIO_H1_START + (_nr))
#define S5PC1XX_GPH2(_nr)	(S5PC1XX_GPIO_H2_START + (_nr))
#define S5PC1XX_GPH3(_nr)	(S5PC1XX_GPIO_H3_START + (_nr))
#define S5PC1XX_GPI(_nr)	(S5PC1XX_GPIO_I_START + (_nr))
#define S5PC1XX_GPJ0(_nr)	(S5PC1XX_GPIO_J0_START + (_nr))
#define S5PC1XX_GPJ1(_nr)	(S5PC1XX_GPIO_J1_START + (_nr))
#define S5PC1XX_GPJ2(_nr)	(S5PC1XX_GPIO_J2_START + (_nr))
#define S5PC1XX_GPJ3(_nr)	(S5PC1XX_GPIO_J3_START + (_nr))
#define S5PC1XX_GPJ4(_nr)	(S5PC1XX_GPIO_J4_START + (_nr))
#define S5PC1XX_MP01(_nr)	(S5PC1XX_GPIO_MP01_START + (_nr))
#define S5PC1XX_MP02(_nr)	(S5PC1XX_GPIO_MP02_START + (_nr))
#define S5PC1XX_MP03(_nr)	(S5PC1XX_GPIO_MP03_START + (_nr))
#define S5PC1XX_MP04(_nr)	(S5PC1XX_GPIO_MP04_START + (_nr))
#define S5PC1XX_MP05(_nr)	(S5PC1XX_GPIO_MP05_START + (_nr))
#define S5PC1XX_MP06(_nr)	(S5PC1XX_GPIO_MP06_START + (_nr))
#define S5PC1XX_MP07(_nr)	(S5PC1XX_GPIO_MP07_START + (_nr))
#define S5PC1XX_MP10(_nr)	(S5PC1XX_GPIO_MP10_START + (_nr))
#define S5PC1XX_MP11(_nr)	(S5PC1XX_GPIO_MP11_START + (_nr))
#define S5PC1XX_MP12(_nr)	(S5PC1XX_GPIO_MP12_START + (_nr))
#define S5PC1XX_MP13(_nr)	(S5PC1XX_GPIO_MP13_START + (_nr))
#define S5PC1XX_MP14(_nr)	(S5PC1XX_GPIO_MP14_START + (_nr))
#define S5PC1XX_MP15(_nr)	(S5PC1XX_GPIO_MP15_START + (_nr))
#define S5PC1XX_MP16(_nr)	(S5PC1XX_GPIO_MP16_START + (_nr))
#define S5PC1XX_MP17(_nr)	(S5PC1XX_GPIO_MP17_START + (_nr))
#define S5PC1XX_MP18(_nr)	(S5PC1XX_GPIO_MP18_START + (_nr))
#define S5PC1XX_MP20(_nr)	(S5PC1XX_GPIO_MP20_START + (_nr))
#define S5PC1XX_MP21(_nr)	(S5PC1XX_GPIO_MP21_START + (_nr))
#define S5PC1XX_MP22(_nr)	(S5PC1XX_GPIO_MP22_START + (_nr))
#define S5PC1XX_MP23(_nr)	(S5PC1XX_GPIO_MP23_START + (_nr))
#define S5PC1XX_MP24(_nr)	(S5PC1XX_GPIO_MP24_START + (_nr))
#define S5PC1XX_MP25(_nr)	(S5PC1XX_GPIO_MP25_START + (_nr))
#define S5PC1XX_MP26(_nr)	(S5PC1XX_GPIO_MP26_START + (_nr))
#define S5PC1XX_MP27(_nr)	(S5PC1XX_GPIO_MP27_START + (_nr))
#define S5PC1XX_MP28(_nr)	(S5PC1XX_GPIO_MP28_START + (_nr))

/* the end of the S5PC1XX specific gpios */
#define S5PC1XX_GPIO_END	(S5PC1XX_MP28(S5PC1XX_GPIO_MP28_NR) + 1)
#define S3C_GPIO_END		S5PC1XX_GPIO_END

/* define the number of gpios we need to the one after the MP04() range */
#define ARCH_NR_GPIOS	(S5PC1XX_MP28(S5PC1XX_GPIO_MP28_NR) + 1)

#else
#error "CPU type should be defined(S5PC100, S5PC110)"
#endif

#include <asm-generic/gpio.h>
